from __future__ import annotations

import inspect
import logging
import os
import sys
from typing import Optional, Tuple

from dict_deep import deep_get, deep_set
from mkdocs.commands.build import DuplicateFilter
from mkdocs.config import config_options as c
from mkdocs.config.base import Config, ConfigErrors, ConfigWarnings
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin
from path import Path

from iconoclast.plugins import new_console

symlink = Path(__file__).parent / ".overrides" / ".icons" / "fontawesome"


class IconokitConfig(Config):
    name = c.Type(str, default="")
    token = c.Type(str, default=os.getenv("FONTAWESOME_API_TOKEN") or "")
    enabled = c.Private()

    def validate(self) -> Tuple[ConfigWarnings, ConfigErrors]:
        warnings, errors = super().validate()
        self.enabled = bool(self.name and self.token)
        return warnings, errors


class IconoclastConfig(Config):
    css = c.Type(bool, default=False)
    kit: IconokitConfig = c.SubConfig(IconokitConfig)


class IconoclastPlugin(BasePlugin[IconoclastConfig]):
    def on_config(self, config: MkDocsConfig) -> Optional[Config]:
        icon_dirs = [symlink]
        css = "iconoclast.min.css"

        symlink.unlink_p()
        symlink.parent.makedirs_p()

        (get_package_path() / "svgs").symlink(symlink)

        if self.config.kit.enabled:
            try:
                import iconokit
            except ImportError:
                with new_console() as console:
                    console.print(
                        "You haven't installed a Font Awesome kit. "
                        "Run [cyan]iconoclast install[/] or change your settings "
                        "for the [magenta]iconoclast[/] plugin.",
                    )
                    log.error(console.export_text(styles=True))
                    sys.exit(1)
            else:
                icon_dirs.append(iconokit.icons())
                css = iconokit.kit("css")

        key = "mdx_configs|pymdownx.emoji|options|custom_icons"

        for icon_dir in icon_dirs:
            custom_icons = (deep_get(config, key, sep="|") or []) + [
                str(icon_dir.parent)
            ]
            deep_set(config, key, custom_icons, sep="|")

            config.theme.dirs.insert(1, str(icon_dir.parent.parent))

        if self.config.css:
            config.extra_css.append(css)

        return config

    def on_post_build(self, *, config: MkDocsConfig) -> None:
        symlink.parent.parent.rmtree_p()

        if self.config.css and not self.config.kit.enabled:
            fa_path = get_package_path()
            site_dir = Path(config.site_dir)

            (fa_path / "css" / "all.min.css").copy(site_dir / "iconoclast.min.css")
            (fa_path / "webfonts").copytree(site_dir / "webfonts")


def get_package_path() -> Path:
    try:
        import fontawesomepro
    except ImportError:
        with new_console() as console:
            console.print(
                "Font Awesome Pro is not installed. Install it or remove the [magenta]iconoclast[/] plugin."
            )
            log.error(console.export_text(styles=True))
            sys.exit(1)
    else:
        return (
            Path(inspect.getfile(fontawesomepro)).parent / "static" / "fontawesomepro"
        )


log = logging.getLogger("mkdocs")
log.addFilter(DuplicateFilter())



import sgqlc.types
import sgqlc.types.datetime

fontawesome_schema = sgqlc.types.Schema()

########################################################################
# Scalars and Enumerations
########################################################################
Boolean = sgqlc.types.Boolean

Date = sgqlc.types.datetime.Date

ID = sgqlc.types.ID

Int = sgqlc.types.Int

String = sgqlc.types.String


########################################################################
# Input Objects
########################################################################
class SubsetIcon(sgqlc.types.Input):
    __schema__ = fontawesome_schema
    __field_names__ = ("name", "styles", "version")
    name = sgqlc.types.Field(String, graphql_name="name")
    styles = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name="styles")
    version = sgqlc.types.Field(String, graphql_name="version")


########################################################################
# Output Objects and Interfaces
########################################################################
class Account(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("email", "id", "kit", "kits", "pro_cdn_referrers")
    email = sgqlc.types.Field(String, graphql_name="email")
    id = sgqlc.types.Field(Int, graphql_name="id")
    kit = sgqlc.types.Field(
        "Kit",
        graphql_name="kit",
        args=sgqlc.types.ArgDict(
            (
                (
                    "token",
                    sgqlc.types.Arg(
                        sgqlc.types.non_null(String), graphql_name="token", default=None
                    ),
                ),
            )
        ),
    )
    kits = sgqlc.types.Field(sgqlc.types.list_of("Kit"), graphql_name="kits")
    pro_cdn_referrers = sgqlc.types.Field(
        "ProCdnReferrers", graphql_name="proCdnReferrers"
    )


class Download(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("separates_web_desktop",)
    separates_web_desktop = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean), graphql_name="separatesWebDesktop"
    )


class FamilyStyle(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("family", "style")
    family = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="family")
    style = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="style")


class FamilyStylesByLicense(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("free", "pro")
    free = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FamilyStyle))),
        graphql_name="free",
    )
    pro = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FamilyStyle))),
        graphql_name="pro",
    )


class Icon(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = (
        "changes",
        "family_styles_by_license",
        "id",
        "label",
        "shim",
        "sris_by_license",
        "unicode",
    )
    changes = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))),
        graphql_name="changes",
    )
    family_styles_by_license = sgqlc.types.Field(
        sgqlc.types.non_null(FamilyStylesByLicense),
        graphql_name="familyStylesByLicense",
    )
    id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="id")
    label = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="label")
    shim = sgqlc.types.Field("Shim", graphql_name="shim")
    sris_by_license = sgqlc.types.Field(
        sgqlc.types.non_null("SrisByLicense"), graphql_name="srisByLicense"
    )
    unicode = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="unicode")


class IconCount(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("free", "pro")
    free = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="free")
    pro = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="pro")


class IconUpload(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("height", "name", "path", "unicode", "version", "width")
    height = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="height")
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")
    path = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="path")
    unicode = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="unicode")
    version = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="version")
    width = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="width")


class Kit(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = (
        "auto_accessibility_enabled",
        "domains",
        "icon_uploads",
        "license_selected",
        "minified",
        "name",
        "release",
        "shim_enabled",
        "status",
        "technology_selected",
        "token",
        "version",
    )
    auto_accessibility_enabled = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean), graphql_name="autoAccessibilityEnabled"
    )
    domains = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))),
        graphql_name="domains",
    )
    icon_uploads = sgqlc.types.Field(
        sgqlc.types.list_of(IconUpload), graphql_name="iconUploads"
    )
    license_selected = sgqlc.types.Field(
        sgqlc.types.non_null(String), graphql_name="licenseSelected"
    )
    minified = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name="minified")
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="name")
    release = sgqlc.types.Field(sgqlc.types.non_null("Release"), graphql_name="release")
    shim_enabled = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean), graphql_name="shimEnabled"
    )
    status = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="status")
    technology_selected = sgqlc.types.Field(
        sgqlc.types.non_null(String), graphql_name="technologySelected"
    )
    token = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name="token")
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="version")


class Membership(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("free", "pro")
    free = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))),
        graphql_name="free",
    )
    pro = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))),
        graphql_name="pro",
    )


class ProCdnReferrers(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("active", "hostnames", "limit")
    active = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name="active")
    hostnames = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))),
        graphql_name="hostnames",
    )
    limit = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name="limit")


class Release(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = (
        "date",
        "download",
        "icon_count",
        "icons",
        "is_latest",
        "sris_by_license",
        "version",
    )
    date = sgqlc.types.Field(sgqlc.types.non_null(Date), graphql_name="date")
    download = sgqlc.types.Field(
        sgqlc.types.non_null(Download), graphql_name="download"
    )
    icon_count = sgqlc.types.Field(
        sgqlc.types.non_null(IconCount), graphql_name="iconCount"
    )
    icons = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Icon))),
        graphql_name="icons",
        args=sgqlc.types.ArgDict(
            (
                (
                    "license",
                    sgqlc.types.Arg(String, graphql_name="license", default=None),
                ),
            )
        ),
    )
    is_latest = sgqlc.types.Field(
        sgqlc.types.non_null(Boolean), graphql_name="isLatest"
    )
    sris_by_license = sgqlc.types.Field(
        sgqlc.types.non_null("SrisByLicense"), graphql_name="srisByLicense"
    )
    version = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="version")


class RootMutationType(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("create_subset",)
    create_subset = sgqlc.types.Field(
        "SubsetZipArchive",
        graphql_name="createSubset",
        args=sgqlc.types.ArgDict(
            (
                (
                    "icons",
                    sgqlc.types.Arg(
                        sgqlc.types.list_of(SubsetIcon),
                        graphql_name="icons",
                        default=None,
                    ),
                ),
                ("id", sgqlc.types.Arg(String, graphql_name="id", default=None)),
                (
                    "version",
                    sgqlc.types.Arg(String, graphql_name="version", default=None),
                ),
            )
        ),
    )


class RootQueryType(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("me", "release", "releases", "search")
    me = sgqlc.types.Field(Account, graphql_name="me")
    release = sgqlc.types.Field(
        Release,
        graphql_name="release",
        args=sgqlc.types.ArgDict(
            (
                (
                    "version",
                    sgqlc.types.Arg(
                        sgqlc.types.non_null(String),
                        graphql_name="version",
                        default=None,
                    ),
                ),
            )
        ),
    )
    releases = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(Release)), graphql_name="releases"
    )
    search = sgqlc.types.Field(
        sgqlc.types.list_of(Icon),
        graphql_name="search",
        args=sgqlc.types.ArgDict(
            (
                ("first", sgqlc.types.Arg(Int, graphql_name="first", default=None)),
                (
                    "query",
                    sgqlc.types.Arg(
                        sgqlc.types.non_null(String), graphql_name="query", default=None
                    ),
                ),
                (
                    "version",
                    sgqlc.types.Arg(
                        sgqlc.types.non_null(String),
                        graphql_name="version",
                        default=None,
                    ),
                ),
            )
        ),
    )


class RootSubscriptionType(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("subset_created",)
    subset_created = sgqlc.types.Field(
        "SubsetZipArchive",
        graphql_name="subsetCreated",
        args=sgqlc.types.ArgDict(
            (("id", sgqlc.types.Arg(String, graphql_name="id", default=None)),)
        ),
    )


class Shim(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("id", "name", "prefix")
    id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="id")
    name = sgqlc.types.Field(String, graphql_name="name")
    prefix = sgqlc.types.Field(String, graphql_name="prefix")


class Sri(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("path", "value")
    path = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="path")
    value = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name="value")


class SrisByLicense(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("free", "pro")
    free = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Sri))),
        graphql_name="free",
    )
    pro = sgqlc.types.Field(
        sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(Sri))),
        graphql_name="pro",
    )


class SubsetZipArchive(sgqlc.types.Type):
    __schema__ = fontawesome_schema
    __field_names__ = ("id", "ready_for_download", "url")
    id = sgqlc.types.Field(String, graphql_name="id")
    ready_for_download = sgqlc.types.Field(Boolean, graphql_name="readyForDownload")
    url = sgqlc.types.Field(String, graphql_name="url")


########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
fontawesome_schema.query_type = RootQueryType
fontawesome_schema.mutation_type = RootMutationType
fontawesome_schema.subscription_type = RootSubscriptionType



import subprocess
from datetime import datetime
from pathlib import Path as StdPath
from tempfile import TemporaryDirectory

import requests
import typer
from dict_deep import deep_get
from halo import Halo
from mkdocs.config.base import load_config
from path import Path
from sgqlc.endpoint.requests import RequestsEndpoint
from sgqlc.operation import Operation
from yarl import URL

from iconoclast.cli.context import set_context
from iconoclast.cli.exceptions import Iconoquit
from iconoclast.cli.graphql.schema import fontawesome_schema as schema
from iconoclast.plugins.iconoclast import IconokitConfig

app = typer.Typer(rich_markup_mode="rich")

here = Path(__file__).parent


@app.command(
    name="install",
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True},
)
@set_context
def install(
    ctx: typer.Context,
    config_file: StdPath = typer.Option(
        None,
        "--config-file",
        exists=True,
        dir_okay=False,
        help="The path to your MkDocs configuration file.",
    ),
):
    """
    Install a kit. This command supports all options of [cyan]pip install[/cyan] in addition to the ones listed
    below.
    """
    config = load_config(str(config_file) if config_file else None)
    kit_config: IconokitConfig = config.plugins["iconoclast"].config.kit

    if not kit_config.enabled:
        raise Iconoquit(
            "You must specify a kit name and Font Awesome API token in "
            "Iconoclast's plugin configuration to use this command."
        )

    api_url = URL("https://api.fontawesome.com")

    resp = requests.post(
        api_url / "token",
        headers={"Authorization": f"Bearer {kit_config.token}"},
    )

    if resp.status_code == 403:
        raise Iconoquit("Your Font Awesome API token is invalid.")
    elif resp.status_code != 200:
        raise Iconoquit("Couldn't communicate with Font Awesome.")

    access_token = resp.json()["access_token"]

    endpoint = RequestsEndpoint(
        str(api_url), base_headers={"Authorization": f"Bearer {access_token}"}
    )

    op = Operation(schema.RootQueryType)
    op.me.kits.name()
    op.me.kits.token()
    op.me.kits.icon_uploads()

    data = endpoint(op)["data"]
    kits = deep_get(data, "me.kits")

    try:
        kit = next(k for k in kits if k["name"] == kit_config.name)
    except StopIteration:
        raise Iconoquit(f'Kit "{kit_config.name}" does not exist')

    icons = kit["iconUploads"]

    with TemporaryDirectory() as tmpdir:
        with Path(tmpdir) as tmp:
            iconokit_root = tmp / "iconokit"
            iconokit_pkg = iconokit_root / "iconokit"

            (here / "iconokit").copytree(iconokit_root)
            (iconokit_pkg / ".token").write_text(kit["token"])

            icons_dir = (
                iconokit_pkg / ".overrides" / ".icons" / "fontawesome" / "kit"
            ).makedirs_p()

            for icon in icons:
                svg = (
                    f"<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 {icon['width']} "
                    f"{icon['height']}\"><path d=\"{icon['path']}\"/></svg>"
                )
                (icons_dir / icon["name"]).with_suffix(".svg").write_text(svg)

            with Halo(
                text=f"Installing {kit_config.name}...", spinner="dots"
            ) as spinner:
                subprocess.run(
                    f"pip install {iconokit_root.abspath()} {' '.join(ctx.args)}".split(),
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.STDOUT,
                )
                spinner.succeed(f"Kit installed!")


@app.callback(
    no_args_is_help=True,
    epilog=f"Iconoclast © {datetime.now().year} celsius narhwal. Thank you kindly for your attention.",
)
def main():
    """
    Iconoclast integrates Font Awesome Pro with Material for MkDocs.
    """



from path import Path


def token() -> str:
    return (Path(__file__).parent / ".token").read_text().strip()


def icons() -> Path:
    return Path(__file__).parent / ".overrides" / ".icons" / "fontawesome"


def kit(mode: str) -> str:
    return f"https://kit.fontawesome.com/{(token())}.{mode}"



from click import ClickException

from iconoclast.cli.context import get_context


class Iconoquit(ClickException):
    def __init__(self, message: str, code: int = 1):
        super().__init__(message.strip())
        setattr(self, "ctx", get_context())
        self.exit_code = code



import logging

import semver
from importlib_metadata import version
from material.plugins.social.plugin import SocialPlugin
from mkdocs.commands.build import DuplicateFilter
from mkdocs.config.defaults import MkDocsConfig
from path import Path

from iconoclast.plugins import new_console


class IconocardsPlugin(SocialPlugin):
    def on_config(self, config):
        public_deprecation = semver.Version.parse(
            "999.999.999"  # placeholder until public version gets rewritten social plugin
        )
        insiders_deprecation = semver.Version.parse("4.33.0")

        theme_version = semver.Version.parse(version("mkdocs-material"))

        public = f"{theme_version.major}.{theme_version.minor}.{theme_version.patch}"
        insiders = getattr(theme_version, "build", "insiders.0.0.0").lstrip("insiders.")

        if public > public_deprecation or insiders > insiders_deprecation:
            with new_console() as console:
                console.print(
                    "The [magenta]iconocards[/] plugin is deprecated for your version of Material for MkDocs. "
                    "Use the built-in social plugin instead."
                )
                log.warning(console.export_text(styles=True))

        return super().on_config(config)

    def _load_logo(self, config: MkDocsConfig):
        theme = config.theme
        icon = theme["icon"]["logo"]

        if icon and "logo" not in theme:
            for path in theme.dirs:
                icon_path = Path(path) / ".icons" / f"{icon}.svg"
                if icon_path.exists():
                    return self._load_logo_svg(icon_path, self.color["text"])

        return super()._load_logo(config)


log = logging.getLogger("mkdocs")
log.addFilter(DuplicateFilter())
